﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Services;
using System.Data;
using SubastasWs.TClassBase;

namespace SubastasWs
{
  /// <summary>
/// Soap Header for the Secured Web Service.
/// Username and Password are required for AuthenticateUser(),
///   and AuthenticatedToken is required for everything else.
/// </summary>
/// 
public class SecuredWebServiceHeader : System.Web.Services.Protocols.SoapHeader
{
  public string Username;
  public string Password;
  public string AuthenticatedToken;
}

/// <summary>
/// Summary description for SecuredWebService
/// </summary>


// To allow this Web Service to be called from script, using ASP.NET AJAX, uncomment the following line. 
// [System.Web.Script.Services.ScriptService]
  /// <summary>
/// Summary description for ServicioSubastas
  /// </summary>
  [WebService(Namespace = "http://tempuri.org/")]
  [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
  [System.ComponentModel.ToolboxItem(false)]
  public class Service1 : System.Web.Services.WebService
  {


    public SecuredWebServiceHeader SoapHeader;


    [WebMethod]
    public TUsuario RegistrarUsuario(TUsuario objUsuario)
    {
      if (objUsuario.Insertar() > 0)
      {

        //Crear un token para las próximas llamadas para es usuario.
        objUsuario.Token = Guid.NewGuid().ToString();

        // añadimos el token en la cache para este usuario un tiempo de expiración de 60 minutos
        HttpRuntime.Cache.Add(objUsuario.Token,objUsuario.NickName, null, System.Web.Caching.Cache.NoAbsoluteExpiration,
        TimeSpan.FromMinutes(60), System.Web.Caching.CacheItemPriority.NotRemovable, null);

        return objUsuario;
      }
      else
        return null;
    }

    [WebMethod]
    [System.Web.Services.Protocols.SoapHeader("SoapHeader")]
    public TUsuario ValidarUsuario()
    {

      if (SoapHeader == null)
        return null;

      if (string.IsNullOrEmpty(SoapHeader.Username) || string.IsNullOrEmpty(SoapHeader.Password))
        return null;

      // Validar usuario en sistema
      TUsuario objUsuario=UsuarioValido(SoapHeader.Username, SoapHeader.Password);
      if (objUsuario == null)
        return null;

      //Crear un token para las próximas llamadas para es usuario.
      objUsuario.Token = Guid.NewGuid().ToString();

      // añadimos el token en la cache para este usuario un tiempo de expiración de 60 minutos
      HttpRuntime.Cache.Add(objUsuario.Token, SoapHeader.Username, null, System.Web.Caching.Cache.NoAbsoluteExpiration,
      TimeSpan.FromMinutes(60),System.Web.Caching.CacheItemPriority.NotRemovable,null);


      // devolvemos el usuario
      return objUsuario;

    }

    private TUsuario UsuarioValido(string strUsuario, string strPassword)
    {
      // validación en el sistema del usuario

      TUsuario objUsuario = new TUsuario();
      if (objUsuario.Validar(strUsuario, strPassword))
        return objUsuario;
      else 
        return null;
    }

    private bool UsuarioValido(SecuredWebServiceHeader SoapHeader)
    {
      if (SoapHeader == null)
        return false;

      // Comprobar si existen el token en la cache
      if (!string.IsNullOrEmpty(SoapHeader.AuthenticatedToken))
        return (HttpRuntime.Cache[SoapHeader.AuthenticatedToken] != null);

      return false;
    }


    [WebMethod]
    public DataSet DimeArticulos()
    {
      TArticulo objArticulo = new TArticulo();

      return objArticulo.CargaDataSet();
    }


    [WebMethod]
    public DataSet BuscarSubastasPorCategorias(int intIDCategoria)
    {
      TSubasta objSubasta = new TSubasta();

      return objSubasta.DimeSubastasActivas(intIDCategoria);
    }

    [WebMethod]
    public DataSet BuscarMisSubastasPorCategorias(int intIDCategoria,int intIDUsuario)
    {
      TSubasta objSubasta = new TSubasta();

      return objSubasta.DimeMisSubastasFinalizadas(intIDCategoria, intIDUsuario);
    }

    [WebMethod]
    public DataSet BuscarSubastasPorNombre(string strNombre)
    {
      TSubasta objSubasta = new TSubasta();

      return objSubasta.DimeSubastasActivas(strNombre);
    }

    [WebMethod]
    public DataSet BuscarCategorias()
    {
      TCategoria objCategoria = new TCategoria();

      return objCategoria.CargaDataSet();
    }


    [WebMethod]
    public TSubasta VerDetalleSubasta(int intIDSubasta)
    {
      TSubasta objSubasta = new TSubasta();

      return objSubasta.DimeSubasta(intIDSubasta);
    }
    

    [WebMethod]
    [System.Web.Services.Protocols.SoapHeader("SoapHeader")]
    public int CrearSubasta(TSubasta objSubasta)
    {
      if (UsuarioValido(SoapHeader))
        return objSubasta.Insertar();
      else
        return -1;
    }



    [WebMethod]
    [System.Web.Services.Protocols.SoapHeader("SoapHeader")]
    public int PujarSubasta(TPuja objPuja)
    {
      if (UsuarioValido(SoapHeader))
        return objPuja.Insertar();
      else
        return -1;
    }

    [WebMethod]
    [System.Web.Services.Protocols.SoapHeader("SoapHeader")]
    public int ComprarAhora(TSubasta objSubasta)
    {
      if (UsuarioValido(SoapHeader))
        return objSubasta.ComprarAhora();
      else
        return -1;
    }

    [WebMethod]
    public TTotalComentario ConsultarComentarios(int intID)
    {
      TTotalComentario objTotalComentario = new TTotalComentario();
      objTotalComentario.ConsultarComentarios(intID);

      return objTotalComentario;
    }

    [WebMethod]
    [System.Web.Services.Protocols.SoapHeader("SoapHeader")]
    public int CrearComentario(TComentario objComentario)
    {
      if (UsuarioValido(SoapHeader))
        return objComentario.CrearComentario();
      else
        return -1;
    }

    [WebMethod]
    public DataSet VerPujas(int intSubasta)
    {
      TPuja objPuja = new TPuja();
      
      return objPuja.ConsultarPujas(intSubasta);
    }

  }
}